"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateUserProfile = exports.changePassword = exports.resetPassword = exports.forgotPassword = exports.VerifyEmail = exports.Login = exports.signUp = exports.authenticate = void 0;
const http_errors_1 = __importDefault(require("http-errors"));
const User_1 = __importDefault(require("../models/User"));
const Verification_1 = __importDefault(require("../models/Verification"));
const bcrypt_1 = __importDefault(require("bcrypt"));
const jsonwebtoken_1 = __importDefault(require("jsonwebtoken"));
const sendEmails_1 = require("../utils/sendEmails");
const signupUtils_1 = require("../utils/signupUtils");
const init_1 = require("../init");
const authenticate = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const config = (0, init_1.getConfig)();
    try {
        const authHeader = req.headers.authorization;
        if (!authHeader || !authHeader.startsWith('Bearer ')) {
            throw (0, http_errors_1.default)(404, 'No token Found');
        }
        const token = authHeader.split(' ')[1];
        const decoded = jsonwebtoken_1.default.verify(token, config.jwtSecret);
        const user = yield User_1.default.findOne({ _id: decoded.id }).exec();
        if (!user || !user.is_verified) {
            throw (0, http_errors_1.default)(404, 'User Not Found');
        }
        const responsez = {
            username: user.username,
            aff: user.aff,
            profile: user.profileImg
        };
        res.status(200).json(responsez);
    }
    catch (err) {
        next(err);
    }
});
exports.authenticate = authenticate;
//sign-up function
const signUp = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const email = req.body.email;
    const passwordRaw = req.body.password;
    let username = req.body.username;
    let profileImg = req.body.image_url;
    try {
        if (!email || !passwordRaw) {
            throw (0, http_errors_1.default)(400, "Parameters missing");
        }
        const isvalidEm = (0, signupUtils_1.checkValidEmail)(email);
        if (!isvalidEm) {
            throw (0, http_errors_1.default)(401, 'Not a valid email that we support as of now');
        }
        const existingUser = yield User_1.default.findOne({ email: email }).exec();
        if (existingUser) {
            if (existingUser.is_verified) {
                throw (0, http_errors_1.default)(409, "User is already regsistered with this email");
            }
            const VerStatus = yield Verification_1.default.findOne({ email: email }).exec();
            if (VerStatus) {
                throw (0, http_errors_1.default)(501, "User is Registred, check your inbox for verification email");
            }
            yield Verification_1.default.create({
                email: email
            });
            yield (0, sendEmails_1.SendMail)(email);
            res.status(200).json({ message: "Verification email sent. Please check your inbox." });
        }
        const passwordHashed = yield bcrypt_1.default.hash(passwordRaw, 10);
        if (!username) {
            username = yield (0, signupUtils_1.generateUniqueUsername)();
        }
        if (!profileImg) {
            profileImg = `https://robohash.org/${username}.png`;
        }
        const aff = (0, signupUtils_1.getAff)(email);
        const newUser = yield User_1.default.create({
            username: username,
            email: email,
            password: passwordHashed,
            is_verified: false,
            profileImg: profileImg,
            aff: aff
        });
        yield Verification_1.default.create({
            email: email
        });
        yield (0, sendEmails_1.SendMail)(email);
        res.status(200).json({ message: "User Registered and Verification email sent. Please check your inbox." });
    }
    catch (error) {
        next(error);
    }
});
exports.signUp = signUp;
const Login = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const email = req.body.email;
    const pass = req.body.password;
    const config = (0, init_1.getConfig)();
    try {
        if (!pass || !email) {
            throw (0, http_errors_1.default)(404, 'Incomplete Credentials');
        }
        const User = yield User_1.default.findOne({ email: email }).exec();
        if (!User || !User.is_verified) {
            throw (0, http_errors_1.default)(401, "User Does not exist or Verified");
        }
        const ismatch = yield bcrypt_1.default.compare(pass, User.password);
        if (!ismatch) {
            throw (0, http_errors_1.default)(404, "Invalid Password");
        }
        const id = User._id;
        const user = {
            username: User.username,
            aff: User.aff,
            profile: User.profileImg
        };
        const token = jsonwebtoken_1.default.sign({ id }, config.jwtSecret, { expiresIn: '336h' });
        res.status(200).json({ user, token });
    }
    catch (error) {
        next(error);
    }
});
exports.Login = Login;
const VerifyEmail = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const { token } = req.params;
    const config = (0, init_1.getConfig)();
    try {
        const decoded = jsonwebtoken_1.default.verify(token, config.jwtSecret);
        const User = yield User_1.default.findOne({ email: decoded.email }).exec();
        if (!User) {
            throw (0, http_errors_1.default)(404, 'No User Registered');
        }
        if (User.is_verified) {
            throw (0, http_errors_1.default)(400, 'User Already Verified');
        }
        const verification = yield Verification_1.default.findOne({ email: decoded.email });
        if (!verification) {
            throw (0, http_errors_1.default)(401, "Token has expired, Request Again Through Signup.");
        }
        User.is_verified = true;
        yield User.save();
        yield Verification_1.default.deleteOne({ email: decoded.email }).exec();
        const id = User._id;
        const user = {
            username: User.username,
            aff: User.aff,
            profile: User.profileImg
        };
        const token_id = jsonwebtoken_1.default.sign({ id }, config.jwtSecret, { expiresIn: '336h' });
        res.status(200).json({ message: "User is Verified, You can now Login", user, token_id });
    }
    catch (error) {
        next(error);
    }
});
exports.VerifyEmail = VerifyEmail;
const forgotPassword = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const email = req.body.email;
    const config = (0, init_1.getConfig)();
    try {
        if (!email) {
            throw (0, http_errors_1.default)(400, "Email is required");
        }
        const user = yield User_1.default.findOne({ email: email, is_verified: true }).exec();
        if (!user) {
            throw (0, http_errors_1.default)(404, "No verified user found with this email");
        }
        const resetToken = jsonwebtoken_1.default.sign({ userId: user._id }, config.jwtSecret, { expiresIn: '1h' });
        yield (0, sendEmails_1.sendResetPasswordEmail)(email, resetToken);
        res.status(200).json({
            message: "Password reset link has been sent to your email"
        });
    }
    catch (error) {
        next(error);
    }
});
exports.forgotPassword = forgotPassword;
const resetPassword = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const { newPassword } = req.body;
    const { token } = req.params;
    const config = (0, init_1.getConfig)();
    try {
        if (!token || !newPassword) {
            throw (0, http_errors_1.default)(400, "Token and new password are required");
        }
        const decoded = jsonwebtoken_1.default.verify(token, config.jwtSecret);
        const user = yield User_1.default.findById(decoded.userId).exec();
        if (!user) {
            throw (0, http_errors_1.default)(404, "User not found");
        }
        const passwordHashed = yield bcrypt_1.default.hash(newPassword, 10);
        user.password = passwordHashed;
        yield user.save();
        res.status(200).json({
            message: "Password has been successfully reset"
        });
    }
    catch (error) {
        if (error instanceof jsonwebtoken_1.default.TokenExpiredError) {
            next((0, http_errors_1.default)(401, "Reset token has expired"));
        }
        else if (error instanceof jsonwebtoken_1.default.JsonWebTokenError) {
            next((0, http_errors_1.default)(401, "Invalid reset token"));
        }
        else {
            next(error);
        }
    }
});
exports.resetPassword = resetPassword;
const changePassword = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const { currentPassword, newPassword } = req.body;
    const authHeader = req.headers.authorization;
    if (!authHeader || !authHeader.startsWith('Bearer ')) {
        throw (0, http_errors_1.default)(404, 'No token Found');
    }
    const token = authHeader.split(' ')[1];
    const decoded = jsonwebtoken_1.default.verify(token, process.env.JWT_SECRET);
    const userId = decoded.id;
    try {
        if (!currentPassword || !newPassword) {
            throw (0, http_errors_1.default)(400, "Current password and new password are required");
        }
        const user = yield User_1.default.findById(userId).exec();
        if (!user) {
            throw (0, http_errors_1.default)(404, "User not found");
        }
        const isMatch = yield bcrypt_1.default.compare(currentPassword, user.password);
        if (!isMatch) {
            throw (0, http_errors_1.default)(401, "Current password is incorrect");
        }
        user.password = yield bcrypt_1.default.hash(newPassword, 10);
        yield user.save();
        res.status(200).json({
            message: "Password has been successfully changed"
        });
    }
    catch (error) {
        next(error);
    }
});
exports.changePassword = changePassword;
const updateUserProfile = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const { username, profileImg } = req.body;
    const authHeader = req.headers.authorization;
    if (!authHeader || !authHeader.startsWith('Bearer ')) {
        throw (0, http_errors_1.default)(404, 'No token Found');
    }
    const token = authHeader.split(' ')[1];
    const decoded = jsonwebtoken_1.default.verify(token, process.env.JWT_SECRET);
    const userId = decoded.id;
    try {
        const user = yield User_1.default.findById(userId).exec();
        if (!user) {
            throw (0, http_errors_1.default)(404, "User not found");
        }
        if (username)
            user.username = username;
        if (profileImg)
            user.profileImg = profileImg;
        yield user.save();
        res.status(200).json({
            message: "Profile has been successfully updated",
            user
        });
    }
    catch (error) {
        next(error);
    }
});
exports.updateUserProfile = updateUserProfile;
