"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sendResetPasswordEmail = exports.SendMail = void 0;
const nodemailer_1 = __importDefault(require("nodemailer"));
const init_1 = require("../init");
const jsonwebtoken_1 = __importDefault(require("jsonwebtoken"));
const SendMail = (email) => __awaiter(void 0, void 0, void 0, function* () {
    const config = (0, init_1.getConfig)();
    const transporter = nodemailer_1.default.createTransport({
        service: 'gmail',
        auth: {
            user: config.emailService.email,
            pass: config.emailService.password
        },
        tls: {
            rejectUnauthorized: false
        }
    });
    const verificationToken = jsonwebtoken_1.default.sign({ email }, config.jwtSecret, { expiresIn: '1h' });
    const verificationUrl = `${config.frontendUrl}/verify_account_email/${verificationToken}`;
    const mailOptions = {
        to: email,
        subject: 'Email Verification',
        text: `Click here to verify your email: ${verificationUrl}`,
        html: `
                <h2>Signup Request</h2>
                <p>To verify your email, click on the link below:</p>
                <a href="${verificationUrl}">Verify Email</a>
                <p>This link will expire in 1 hour.</p>
                <p>If you did not request this, please ignore this email.</p>
        `
    };
    yield transporter.sendMail(mailOptions);
});
exports.SendMail = SendMail;
const sendResetPasswordEmail = (email, resetToken) => __awaiter(void 0, void 0, void 0, function* () {
    const config = (0, init_1.getConfig)();
    const transporter = nodemailer_1.default.createTransport({
        service: 'gmail',
        auth: {
            user: config.emailService.email,
            pass: config.emailService.password
        },
        tls: {
            rejectUnauthorized: false
        }
    });
    const resetUrl = `${config.frontendUrl}/reset_password/${resetToken}`;
    const mailOptions = {
        to: email,
        subject: 'Password Reset Request',
        text: `To reset your password, click on this link: ${resetUrl}\n\nThis link will expire in 1 hour.\n\nIf you did not request this, please ignore this email.`,
        html: `
            <h2>Password Reset Request</h2>
            <p>To reset your password, click on the link below:</p>
            <a href="${resetUrl}">Reset Password</a>
            <p>This link will expire in 1 hour.</p>
            <p>If you did not request this, please ignore this email.</p>
        `
    };
    yield transporter.sendMail(mailOptions);
});
exports.sendResetPasswordEmail = sendResetPasswordEmail;
