"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAff = exports.checkValidEmail = void 0;
exports.generateUniqueUsername = generateUniqueUsername;
const unique_names_generator_1 = require("unique-names-generator");
const User_1 = __importDefault(require("../models/User"));
const init_1 = require("../init");
function generateUniqueUsername() {
    return __awaiter(this, void 0, void 0, function* () {
        const allDictionaries = [unique_names_generator_1.adjectives, unique_names_generator_1.animals, unique_names_generator_1.colors, unique_names_generator_1.countries, unique_names_generator_1.names, unique_names_generator_1.starWars];
        const maxRetries = 4;
        let retries = 0;
        const config = (0, init_1.getConfig)();
        while (retries < maxRetries) {
            const selectedDictionaries = [];
            while (selectedDictionaries.length < 3) {
                const randomDict = allDictionaries[Math.floor(Math.random() * allDictionaries.length)];
                if (!selectedDictionaries.includes(randomDict)) {
                    selectedDictionaries.push(randomDict);
                }
            }
            const seperators = ['-', '_', ''];
            const config = {
                dictionaries: selectedDictionaries,
                separator: seperators[Math.floor(Math.random() * seperators.length)],
                length: 3,
            };
            const newUsername = (0, unique_names_generator_1.uniqueNamesGenerator)(config);
            const existingUser = yield User_1.default.findOne({ username: newUsername }).select('username');
            if (!existingUser) {
                return newUsername;
            }
            retries++;
        }
        const fallbackUsername = (0, unique_names_generator_1.uniqueNamesGenerator)({
            dictionaries: [unique_names_generator_1.adjectives, unique_names_generator_1.animals, unique_names_generator_1.colors],
            separator: '_',
            length: 3,
        });
        const randomNumber = Math.floor(Math.random() * 10000);
        return `${fallbackUsername}_${randomNumber}`;
    });
}
const checkValidEmail = (email) => {
    const config = (0, init_1.getConfig)();
    const valid_email_domains = config.validDomains;
    const match = email.match(/@([\w.-]+)$/);
    if (!match) {
        return false;
    }
    const domain = match[1];
    if (valid_email_domains.has(domain)) {
        return true;
    }
    return false;
};
exports.checkValidEmail = checkValidEmail;
const getAff = (email) => {
    const config = (0, init_1.getConfig)();
    const valid_email_domains = config.validDomains;
    const match = email.match(/@([\w.-]+)$/);
    if (!match) {
        return null;
    }
    const domain = match[1];
    if (valid_email_domains.has(domain)) {
        return valid_email_domains.get(domain);
    }
    return null;
};
exports.getAff = getAff;
